@@ -28,7 +28,7 @@ static int open(struct net_device *dev)
 	int ret;
 	struct wireguard_peer *peer, *temp;
 	struct wireguard_device *wg = netdev_priv(dev);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
+#ifndef COMPAT_CANNOT_USE_IN6_DEV_GET
 	struct inet6_dev *dev_v6 = __in6_dev_get(dev);
 #endif
 	struct in_device *dev_v4 = __in_dev_get_rtnl(dev);
@@ -41,12 +41,12 @@ static int open(struct net_device *dev)
 		IN_DEV_CONF_SET(dev_v4, SEND_REDIRECTS, false);
 		IPV4_DEVCONF_ALL(dev_net(dev), SEND_REDIRECTS) = false;
 	}
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 17, 0)
+#ifndef COMPAT_CANNOT_USE_IN6_DEV_GET
 	if (dev_v6)
-#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 11, 0)
-		dev_v6->addr_gen_mode = IN6_ADDR_GEN_MODE_NONE;
-#else
+#ifndef COMPAT_CANNOT_USE_DEV_CNF
 		dev_v6->cnf.addr_gen_mode = IN6_ADDR_GEN_MODE_NONE;
+#else
+		dev_v6->addr_gen_mode = IN6_ADDR_GEN_MODE_NONE;
 #endif
 #endif
 
@@ -81,7 +81,6 @@ static int suspending_clear_noise_peers(struct notifier_block *nb, unsigned long
 	}
 	rtnl_unlock();
 	rcu_barrier_bh();
-
 	return 0;
 }
 static struct notifier_block clear_peers_on_suspend = { .notifier_call = suspending_clear_noise_peers };
@@ -91,6 +90,7 @@ static int stop(struct net_device *dev)
 {
 	struct wireguard_device *wg = netdev_priv(dev);
 	struct wireguard_peer *peer, *temp;
+
 	peer_for_each (wg, peer, temp, true) {
 		skb_queue_purge(&peer->staged_packet_queue);
 		timers_stop(peer);
@@ -108,14 +108,9 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 	struct wireguard_peer *peer;
 	struct sk_buff *next;
 	struct sk_buff_head packets;
+	sa_family_t family;
 	int ret;
 
-	if (unlikely(dev_recursion_level() > 4)) {
-		ret = -ELOOP;
-		net_dbg_ratelimited("%s: Routing loop detected\n", dev->name);
-		goto err;
-	}
-
 	if (unlikely(skb_examine_untrusted_ip_hdr(skb) != skb->protocol)) {
 		ret = -EPROTONOSUPPORT;
 		net_dbg_ratelimited("%s: Invalid IP packet\n", dev->name);
@@ -129,10 +124,8 @@ static netdev_tx_t xmit(struct sk_buff *skb, struct net_device *dev)
 		goto err;
 	}
 
-	read_lock_bh(&peer->endpoint_lock);
-	ret = peer->endpoint.addr.sa_family != AF_INET && peer->endpoint.addr.sa_family != AF_INET6;
-	read_unlock_bh(&peer->endpoint_lock);
-	if (unlikely(ret)) {
+	family = READ_ONCE(peer->endpoint.addr.sa_family);
+	if (unlikely(family != AF_INET && family != AF_INET6)) {
 		ret = -EDESTADDRREQ;
 		net_dbg_ratelimited("%s: No valid endpoint has been configured or discovered for peer %Lu\n", dev->name, peer->internal_id);
 		goto err_peer;
@@ -209,10 +202,10 @@ static void destruct(struct net_device *dev)
 	wg->incoming_port = 0;
 	destroy_workqueue(wg->handshake_receive_wq);
 	destroy_workqueue(wg->handshake_send_wq);
-	free_percpu(wg->decrypt_queue.worker);
-	free_percpu(wg->encrypt_queue.worker);
+	packet_queue_free(&wg->decrypt_queue, true);
+	packet_queue_free(&wg->encrypt_queue, true);
 	destroy_workqueue(wg->packet_crypt_wq);
-	routing_table_free(&wg->peer_routing_table);
+	routing_table_free(&wg->peer_routing_table, &wg->device_update_lock);
 	ratelimiter_uninit();
 	memzero_explicit(&wg->static_identity, sizeof(struct noise_static_identity));
 	skb_queue_purge(&wg->incoming_handshakes);
@@ -238,7 +231,7 @@ static void setup(struct net_device *dev)
 	dev->needed_tailroom = noise_encrypted_len(MESSAGE_PADDING_MULTIPLE);
 	dev->type = ARPHRD_NONE;
 	dev->flags = IFF_POINTOPOINT | IFF_NOARP;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 3, 0)
+#ifndef COMPAT_CANNOT_USE_IFF_NO_QUEUE
 	dev->priv_flags |= IFF_NO_QUEUE;
 #else
 	dev->tx_queue_len = 0;
@@ -293,10 +286,10 @@ static int newlink(struct net *src_net, struct net_device *dev, struct nlattr *t
 	if (!wg->packet_crypt_wq)
 		goto error_5;
 
-	if (packet_queue_init(&wg->encrypt_queue, packet_encrypt_worker, true) < 0)
+	if (packet_queue_init(&wg->encrypt_queue, packet_encrypt_worker, true, MAX_QUEUED_PACKETS) < 0)
 		goto error_6;
 
-	if (packet_queue_init(&wg->decrypt_queue, packet_decrypt_worker, true) < 0)
+	if (packet_queue_init(&wg->decrypt_queue, packet_decrypt_worker, true, MAX_QUEUED_PACKETS) < 0)
 		goto error_7;
 
 	ret = ratelimiter_init();
@@ -319,9 +312,9 @@ static int newlink(struct net *src_net, struct net_device *dev, struct nlattr *t
 error_9:
 	ratelimiter_uninit();
 error_8:
-	free_percpu(wg->decrypt_queue.worker);
+	packet_queue_free(&wg->decrypt_queue, true);
 error_7:
-	free_percpu(wg->encrypt_queue.worker);
+	packet_queue_free(&wg->encrypt_queue, true);
 error_6:
 	destroy_workqueue(wg->packet_crypt_wq);
 error_5:
@@ -348,6 +341,7 @@ int __init device_init(void)
 {
 #ifdef CONFIG_PM_SLEEP
 	int ret = register_pm_notifier(&clear_peers_on_suspend);
+
 	if (ret)
 		return ret;
 #endif
