@@ -271,6 +271,8 @@ ip2 link add dev wg0 type wireguard
 configure_peers
 ip1 link add veth1 type veth peer name veth2
 ip1 link set veth2 netns $netns2
+n1 bash -c 'printf 0 > /proc/sys/net/ipv6/conf/all/accept_dad'
+n2 bash -c 'printf 0 > /proc/sys/net/ipv6/conf/all/accept_dad'
 n1 bash -c 'printf 0 > /proc/sys/net/ipv6/conf/veth1/accept_dad'
 n2 bash -c 'printf 0 > /proc/sys/net/ipv6/conf/veth2/accept_dad'
 n1 bash -c 'printf 1 > /proc/sys/net/ipv4/conf/veth1/promote_secondaries'
@@ -376,7 +378,7 @@ ip0 link add dev wg0 type wireguard
 config=( "[Interface]" "PrivateKey=$(wg genkey)" "[Peer]" "PublicKey=$(wg genkey)" )
 for a in {1..255}; do
 	for b in {0..255}; do
-		config+=( "AllowedIPs=$a.$b.0.0/16" )
+		config+=( "AllowedIPs=$a.$b.0.0/16,$a::$b/128" )
 	done
 done
 n0 wg setconf wg0 <(printf '%s\n' "${config[@]}")
@@ -384,7 +386,7 @@ i=0
 for ip in $(n0 wg show wg0 allowed-ips); do
 	((++i))
 done
-((i == 65281))
+((i == 255*256*2+1))
 ip0 link del wg0
 ip0 link add dev wg0 type wireguard
 config=( "[Interface]" "PrivateKey=$(wg genkey)" )
